unit adxOLBuiltInBug_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, adxAddIn, adxOLBuiltInBug_TLB,
  StdVcl, Classes, Contnrs, Outlook2000;

type
  TADX_OLBuiltInBug = class(TadxAddin, IADX_OLBuiltInBug)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    adxOlInspectorCommandBar1: TadxOlInspectorCommandBar;
    adxBuiltInControl1: TadxBuiltInControl;
    adxOlInspectorCommandBar2: TadxOlInspectorCommandBar;
    procedure adxBuiltInControl1Action(Sender: TObject);
    procedure adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
    procedure adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
    procedure adxCOMAddInModuleOLNewInspector(ASender: TObject;
      const Inspector: _Inspector);
  private
    FInboxItems: TItems;
    FDeletedItems: TItems;
    FSentItems: TItems;
    FMailList: TObjectList;
  protected
  public
    procedure DoItemSend(Sender: TObject; var Item: OleVariant; var Cancel: OleVariant);
    procedure DoItemAdd(Sender: TObject; var Item: OleVariant);

    procedure DoOpen(Sender: TObject; var Cancel: OleVariant);
    procedure DoClose(Sender: TObject; var Cancel: OleVariant);
    procedure DoForward(Sender: TObject; var Forward: OleVariant; var Cancel: OleVariant);
    procedure DoReply(Sender: TObject; var Response: OleVariant; var Cancel: OleVariant);
    procedure DoReplyAll(Sender: TObject; var Response: OleVariant; var Cancel: OleVariant);
    procedure DoCustomPropertyChange(Sender: TObject; var Name: OleVariant);
  end;

implementation

{$R *.dfm}

uses Dialogs;

procedure TAddInModule.adxBuiltInControl1Action(Sender: TObject);
begin
  ShowMessage('Click');
end;

procedure TAddInModule.adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
var
  IFolder: MAPIFolder;
begin
  FMailList := TObjectList.Create(False);
  OutlookApp.OnItemSend := DoItemSend;

  IFolder := OutlookApp.GetNameSpace('MAPI').GetDefaultFolder(olFolderInbox);
  if Assigned(IFolder) then begin
    FInboxItems := TItems.Create(nil);
    FInboxItems.ConnectTo(IFolder.Items);
    FInboxItems.OnItemAdd := DoItemAdd;
  end;
  IFolder := OutlookApp.GetNameSpace('MAPI').GetDefaultFolder(olFolderDeletedItems);
  if Assigned(IFolder) then begin
    FDeletedItems := TItems.Create(nil);
    FDeletedItems.ConnectTo(IFolder.Items);
    FDeletedItems.OnItemAdd := DoItemAdd;
  end;
  IFolder := OutlookApp.GetNameSpace('MAPI').GetDefaultFolder(olFolderSentMail);
  if Assigned(IFolder) then begin
    FSentItems := TItems.Create(nil);
    FSentItems.ConnectTo(IFolder.Items);
    FSentItems.OnItemAdd := DoItemAdd;
  end;
end;

procedure TAddInModule.adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
var
  i: Integer;
begin
  if Assigned(FSentItems) then
    FSentItems.Disconnect;
  if Assigned(FDeletedItems) then
    FDeletedItems.Disconnect;
  if Assigned(FInboxItems) then
    FInboxItems.Disconnect;
  for i := 0 to FMailList.Count - 1 do
    TMailItem(FMailList[i]).Free;
  FMailList.Free;
end;

procedure TAddInModule.adxCOMAddInModuleOLNewInspector(ASender: TObject; const Inspector: _Inspector);
var
  IDsp: IDispatch;
  IMail: MailItem;
  MailItem: TMailItem;
begin
  IDsp := Inspector.CurrentItem;
  if Assigned(IDsp) then
    try
      IDsp.QueryInterface(IID__MailItem, IMail);
      if Assigned(IMail) then
        try
          MailItem := TMailItem.Create(nil);
          MailItem.ConnectTo(IMail);
          MailItem.OnOpen := DoOpen;
          MailItem.OnClose := DoClose;
          MailItem.OnForward := DoForward;
          MailItem.OnReply := DoReply;
          MailItem.OnReplyAll := DoReplyAll;
          MailItem.OnCustomPropertyChange := DoCustomPropertyChange;
          FMailList.Add(MailItem);
        finally
          IMail := nil;
        end;
    finally
      IDsp := nil;
    end;
end;

procedure TAddInModule.DoItemSend(Sender: TObject; var Item,
  Cancel: OleVariant);
begin
  // do nothing
end;

procedure TAddInModule.DoItemAdd(Sender: TObject; var Item: OleVariant);
begin
  // do nothing
end;

procedure TAddInModule.DoOpen(Sender: TObject; var Cancel: OleVariant);
begin
  // do nothing
end;

procedure TAddInModule.DoClose(Sender: TObject; var Cancel: OleVariant);
var
  Index: Integer;
begin
  Index := FMailList.IndexOf(Sender);
  if Index >= 0 then begin
    TMailItem(FMailList[Index]).Free;
    FMailList.Delete(Index);
  end;
end;

procedure TAddInModule.DoForward(Sender: TObject; var Forward: OleVariant; var Cancel: OleVariant);
begin
  // do nothing
end;

procedure TAddInModule.DoReply(Sender: TObject; var Response: OleVariant; var Cancel: OleVariant);
begin
  // do nothing
end;

procedure TAddInModule.DoReplyAll(Sender: TObject; var Response: OleVariant; var Cancel: OleVariant);
begin
  // do nothing
end;

procedure TAddInModule.DoCustomPropertyChange(Sender: TObject; var Name: OleVariant);
begin
  // do nothing
end;

initialization
  TadxFactory.Create(ComServer, TADX_OLBuiltInBug, CLASS_ADX_OLBuiltInBug, TAddInModule);

end.
